@@ -1,10 +1,13 @@
 from zebrazoom.code.preprocessImage import preprocessImage
 import numpy as np
 import cv2
+import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 
 def getForegroundImage(videoPath, background, frameNumber, wellNumber, wellPositions, hyperparameters):
   
   minPixelDiffForBackExtract = hyperparameters["minPixelDiffForBackExtract"]
+  if "minPixelDiffForBackExtractHead" in hyperparameters:
+    minPixelDiffForBackExtract = hyperparameters["minPixelDiffForBackExtractHead"]
   debug = 0
   
   if len(wellPositions):
@@ -20,7 +23,7 @@ def getForegroundImage(videoPath, background, frameNumber, wellNumber, wellPosit
   
   back = background[ytop:ytop+lenY, xtop:xtop+lenX]
   
-  cap = cv2.VideoCapture(videoPath)
+  cap = zzVideoReading.VideoCapture(videoPath)
   cap.set(1, frameNumber)
   ret, frame = cap.read()
   
@@ -37,17 +40,19 @@ def getForegroundImage(videoPath, background, frameNumber, wellNumber, wellPosit
     
   grey = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
   curFrame = grey[ytop:ytop+lenY, xtop:xtop+lenX]
+  initialCurFrame = curFrame.copy()
   
   # if False:
   putToWhite = ( curFrame.astype('int32') >= (back.astype('int32') - minPixelDiffForBackExtract) )
   
   curFrame[putToWhite] = 255
-  
+  previousNbBlackPixels = []
   if hyperparameters["adjustMinPixelDiffForBackExtract_nbBlackPixelsMax"]:
+    minPixel2nbBlackPixels = {}
     countTries = 0
     nbBlackPixels = 0
     nbBlackPixelsMax = int(hyperparameters["adjustMinPixelDiffForBackExtract_nbBlackPixelsMax"])
-    while ((nbBlackPixels < 1) or (nbBlackPixels > nbBlackPixelsMax)) and (countTries < 30):
+    while (minPixelDiffForBackExtract > 0) and (countTries < 30) and not(minPixelDiffForBackExtract in minPixel2nbBlackPixels):
       if countTries > 0:
         curFrame = grey[ytop:ytop+lenY, xtop:xtop+lenX]
         putToWhite = ( curFrame.astype('int32') >= (back.astype('int32') - minPixelDiffForBackExtract) )
@@ -55,12 +60,29 @@ def getForegroundImage(videoPath, background, frameNumber, wellNumber, wellPosit
       ret, thresh1 = cv2.threshold(curFrame, hyperparameters["thresholdForBlobImg"], 255, cv2.THRESH_BINARY)
       thresh1 = 255 - thresh1
       nbBlackPixels = cv2.countNonZero(thresh1)
+      minPixel2nbBlackPixels[minPixelDiffForBackExtract] = nbBlackPixels
       if nbBlackPixels > nbBlackPixelsMax:
         minPixelDiffForBackExtract = minPixelDiffForBackExtract + 1
-      if nbBlackPixels < 1 and minPixelDiffForBackExtract > 1:
+      if nbBlackPixels <= nbBlackPixelsMax:
         minPixelDiffForBackExtract = minPixelDiffForBackExtract - 1
       countTries = countTries + 1
-    # print("countTries:", countTries, "; nbBlackPixels:", nbBlackPixels, "; minPixelDiffForBackExtract:", minPixelDiffForBackExtract)
+      previousNbBlackPixels.append(nbBlackPixels)
+      if len(previousNbBlackPixels) >= 3:
+        lastThree = previousNbBlackPixels[len(previousNbBlackPixels)-3: len(previousNbBlackPixels)]
+        if lastThree.count(lastThree[0]) == len(lastThree):
+          countTries = 1000000
+    
+    best_minPixelDiffForBackExtract = 0
+    minDist = 10000000000000
+    for minPixelDiffForBackExtract in minPixel2nbBlackPixels:
+      nbBlackPixels = minPixel2nbBlackPixels[minPixelDiffForBackExtract]
+      dist = abs(nbBlackPixels - nbBlackPixelsMax)
+      if dist < minDist:
+        minDist = dist
+        best_minPixelDiffForBackExtract = minPixelDiffForBackExtract
+        
+    minPixelDiffForBackExtract = best_minPixelDiffForBackExtract
+    hyperparameters["minPixelDiffForBackExtractHead"] = minPixelDiffForBackExtract
     curFrame = grey[ytop:ytop+lenY, xtop:xtop+lenX]
     putToWhite = ( curFrame.astype('int32') >= (back.astype('int32') - minPixelDiffForBackExtract) )
     curFrame[putToWhite] = 255      
@@ -82,4 +104,4 @@ def getForegroundImage(videoPath, background, frameNumber, wellNumber, wellPosit
     cv2.imshow('Frame', frame)
     cv2.waitKey(0)
     
-  return curFrame
+  return [curFrame, initialCurFrame, back]
